from .tools import toolkit
from .prompts import system_message

import tempfile
from openai import OpenAI
from dotenv import load_dotenv
from langchain_openai import ChatOpenAI
from langchain_core.messages import SystemMessage, HumanMessage

load_dotenv()


class IVA:
    def __init__(self):
        self.llm = ChatOpenAI(
            model="gpt-4o-mini",
            temperature=0,
        ).bind_tools(toolkit)

        self.openai_client = OpenAI()

    def transcript(self, file):
        audio = open(file, "rb")
        transcript = self.openai_client.audio.transcriptions.create(
            model="whisper-1",
            file=audio,
            language="ro",
        )

        print(f"Transcript: {transcript.text}")
        return transcript.text

    def process_query(self, query):
        messages = [
            SystemMessage(system_message),
            HumanMessage(query),
        ]

        response = self.llm.invoke(messages)
        messages.append(response)

        for tool_call in response.tool_calls:
            selected_tool = {
                "select_and_get_sensor_value": toolkit[0],
            }[tool_call["name"].lower()]

            tool_msg = selected_tool.invoke(tool_call)
            messages.append(tool_msg)

        final_response = self.llm.invoke(messages)

        return final_response.content

    def tts(self, text):
        response = self.openai_client.audio.speech.create(
            model="tts-1",
            voice="nova",
            input=text,
        )

        with tempfile.NamedTemporaryFile(suffix=".mp3", delete=False) as temp_file:
            temp_file.write(response.content)

        return temp_file.name
